@@ -5,14 +5,14 @@
 	tape drive.
 
 	Maintained by Kai Mkisara (email Kai.Makisara@metla.fi)
-	Copyright by Kai Mkisara, 1998 - 2000. The program may be distributed
+	Copyright by Kai Mkisara, 1998 - 2001. The program may be distributed
 	according to the GNU Public License
 
-	Last Modified: Thu Nov 30 21:15:38 2000 by makisara@kai.makisara.local
+	Last Modified: Wed Nov 21 23:14:29 2001 by makisara@kai.makisara.local
 */
 
 #ifndef lint
-static char rcsid[] = "$Id: /usr2/users/makisara/src/sys/mt-st-0.6/mt.c at Thu Nov 30 21:15:38 2000 by makisara@kai.makisara.local$";
+static char rcsid[] = "$Id: /home/makisara/src/sys/mt-st-0.7/mt.c at Wed Nov 21 23:14:29 2001 by makisara@kai.makisara.local$";
 #endif
 
 #include <stdio.h>
@@ -31,7 +31,7 @@ static char rcsid[] = "$Id: /usr2/users/makisara/src/sys/mt-st-0.6/mt.c at Thu N
 #define DEFTAPE "/dev/tape"     /* default tape device */
 #endif /* DEFTAPE */
 
-#define VERSION "0.6"
+#define VERSION "0.7"
 
 typedef int (* cmdfunc)(/* int, struct cmdef_tr *, int, char ** */);
 
@@ -101,6 +101,8 @@ static cmdef_tr cmds[] = {
     ET_ONLINE },
     { "rewoffl",	MTOFFL,	  do_standard, 0, FD_RDONLY, NO_ARGS,
     ET_ONLINE },
+    { "eject",		MTOFFL,	  do_standard, 0, FD_RDONLY, NO_ARGS,
+    ET_ONLINE },
     { "retension",	MTRETEN,  do_standard, 0, FD_RDONLY, NO_ARGS,
     ET_ONLINE },
     { "eod",		MTEOM,	  do_standard, 0, FD_RDONLY, NO_ARGS,
@@ -113,24 +115,18 @@ static cmdef_tr cmds[] = {
     ET_ONLINE },
     { "status",		MTNOP,	  do_status,   0, FD_RDONLY, NO_ARGS,
     0 },
-    { "erase",		MTERASE,  do_standard, 0, FD_RDWR,   NO_ARGS,
+    { "erase",		MTERASE,  do_standard, 0, FD_RDWR,   ONE_ARG,
     ET_ONLINE },
     { "setblk",		MTSETBLK, do_standard, 0, FD_RDONLY, ONE_ARG,
     0 },
-#ifdef MTLOCK
     { "lock",		MTLOCK,   do_standard, 0, FD_RDONLY, NO_ARGS,
     ET_ONLINE },
     { "unlock", 	MTUNLOCK, do_standard, 0, FD_RDONLY, NO_ARGS,
     ET_ONLINE },
-#endif
-#ifdef MTLOAD
-    { "load",		MTLOAD,	  do_standard, 0, FD_RDONLY, NO_ARGS,
+    { "load",		MTLOAD,	  do_standard, 0, FD_RDONLY, ONE_ARG,
     0 },
-#endif
-#ifdef MTCOMPRESSION
     { "compression",	MTCOMPRESSION,  do_standard,  0, FD_RDONLY, ONE_ARG,
     0 },
-#endif
     { "setdensity",	MTSETDENSITY,   do_standard,  0, FD_RDONLY, ONE_ARG,
     0 },
     { "drvbuffer",	MTSETDRVBUFFER, do_drvbuffer, 0, FD_RDONLY, ONE_ARG,
@@ -139,7 +135,6 @@ static cmdef_tr cmds[] = {
       FD_RDONLY, ONE_ARG, 0},
     { "stoptions",	DO_BOOLEANS,    do_options,   0, FD_RDONLY,
       MULTIPLE_ARGS, 0},
-#ifdef MT_ST_SETBOOLEANS
     { "stsetoptions",   SET_BOOLEANS,   do_options,   0, FD_RDONLY,
       MULTIPLE_ARGS, 0},
     { "stclearoptions", CLEAR_BOOLEANS, do_options,   0, FD_RDONLY,
@@ -152,23 +147,20 @@ static cmdef_tr cmds[] = {
       FD_RDONLY, ONE_ARG, 0},
     { "defcompression", MTSETDRVBUFFER, do_drvbuffer, MT_ST_DEF_COMPRESSION,
       FD_RDONLY, ONE_ARG, 0},
-#endif
-#ifdef MT_ST_SET_TIMEOUT
+    { "stsetcln",	MTSETDRVBUFFER, do_drvbuffer, MT_ST_SET_CLN,
+      FD_RDONLY, ONE_ARG, 0},
     { "sttimeout",	MTSETDRVBUFFER, do_drvbuffer, MT_ST_SET_TIMEOUT,
       FD_RDONLY, ONE_ARG, 0},
     { "stlongtimeout",	MTSETDRVBUFFER, do_drvbuffer, MT_ST_SET_LONG_TIMEOUT,
       FD_RDONLY, ONE_ARG, 0},
-#endif
     { "densities",	0, print_densities,     0, NO_FD,     NO_ARGS,
     0 },
-#ifdef MTSETPART
     { "setpartition",	MTSETPART, do_standard, 0, FD_RDONLY, ONE_ARG,
     ET_ONLINE },
     { "mkpartition",	MTMKPART,  do_standard, 0, FD_RDWR,   ONE_ARG,
     ET_ONLINE },
     { "partseek",	0,         do_partseek, 0, FD_RDONLY, TWO_ARGS,
     ET_ONLINE },
-#endif
     { "asf",		0,         do_asf, MTREW,  FD_RDONLY, ONE_ARG,
     ET_ONLINE },
     { NULL, 0, 0, 0 }
@@ -224,15 +216,18 @@ static struct densities {
     {0x31, "AIT-2"},
     {0x33, "SLR6"},
     {0x34, "SLR100"},
+    {0x40, "DLT1 40 GB, or Ultrium"},
     {0x41, "DLT 40GB"},
     {0x45, "QIC-3095-MC (TR-4)"},
     {0x47, "TR-5"},
-    {0x80, "DLT 15GB uncompressed"},
+    {0x80, "DLT 15GB uncomp. or Ecrix"},
     {0x81, "DLT 15GB compressed"},
     {0x82, "DLT 20GB uncompressed"},
     {0x83, "DLT 20GB compressed"},
     {0x84, "DLT 35GB uncompressed"},
     {0x85, "DLT 35GB compressed"},
+    {0x86, "DLT1 40 GB uncompressed"},
+    {0x87, "DLT1 40 GB compressed"},
     {0x88, "DLT 40GB uncompressed"},
     {0x89, "DLT 40GB compressed"},
     {0x8c, "EXB-8505 compressed"},
@@ -258,6 +253,7 @@ static struct booleans {
     {"no-blklimits",  MT_ST_NO_BLKLIMS,    "drive doesn't support read block limits"},
     {"can-partitions",MT_ST_CAN_PARTITIONS,"drive can handle partitioned tapes"},
     {"scsi2logical",  MT_ST_SCSI2LOGICAL,  "logical block addresses used with SCSI-2"},
+    {"no-wait",       MT_ST_NOWAIT,        "immediate mode for rewind, etc."},
 #ifdef MT_ST_SYSV
     {"sysv",	      MT_ST_SYSV,	   "enable the SystemV semantics"},
 #endif
@@ -659,6 +655,8 @@ do_status(int mtfd, cmdef_tr *cmd, int argc, char **argv)
 	printf(" DR_OPEN");	  
     if (GMT_IM_REP_EN(status.mt_gstat))
 	printf(" IM_REP_EN");
+    if (GMT_CLN(status.mt_gstat))
+	printf(" CLN");
     printf("\n");
     return 0;
 }
