@@ -1,15 +1,15 @@
 /* This program initializes Linux SCSI tape drives using the
    inquiry data from the devices and a text database.
 
-   Copyright 1996-2000 by Kai Mkisara (email Kai.Makisara@metla.fi)
+   Copyright 1996-2001 by Kai Mkisara (email Kai.Makisara@metla.fi)
    Distribution of this program is allowed according to the
    GNU Public Licence.
 
-   Last modified: Mon Nov 27 21:17:39 2000 by makisara@kai.makisara.local
+   Last modified: Thu Nov  8 21:13:48 2001 by makisara@kai.makisara.local
 */
 
 #ifndef lint
-static char rcsid[] = "$Id: /home/makisara/src/sys/mt-st-0.6/stinit.c at Mon Nov 27 21:17:39 2000 by makisara@kai.makisara.local$";
+static char rcsid[] = "$Id: /usr2/users/makisara/src/sys/mt-st-0.7/stinit.c at Thu Nov  8 21:13:48 2001 by makisara@kai.makisara.local$";
 #endif
 
 #include <stdio.h>
@@ -33,7 +33,7 @@ static char rcsid[] = "$Id: /home/makisara/src/sys/mt-st-0.6/stinit.c at Mon Nov
 #endif
 #define SKIP_WHITE(p) for ( ; *p == ' ' || *p == '\t'; p++)
 
-#define VERSION "0.6"
+#define VERSION "0.7"
 
 typedef struct _modepar_tr {
     int defined;
@@ -59,6 +59,8 @@ typedef struct _devdef_tr {
     int drive_buffering;
     int timeout;
     int long_timeout;
+    int cleaning;
+    int nowait;
     modepar_tr modedefs[4];
 } devdef_tr;
 
@@ -73,11 +75,23 @@ typedef struct _devdef_tr {
 static int verbose = 0;
 
 /* The device directories being searched */
-static char *devdirs[] = {"/dev/scsi", "/dev", NULL};
+typedef struct 
+{
+    const char *dir;
+    int selective_scan;
+} devdir;
+static devdir devdirs[] = { {"/dev/scsi", 0}, {"/dev", 1}, {NULL, 0}};
+
+#define DEVFS_PATH    "/dev/tapes"
+#define DEVFS_TAPEFMT DEVFS_PATH "/tape%d"
+
+/* The partial names of the tape devices being included in the
+   search in selective scan */
+static char *tape_name_bases[] = {
+    "st", "nst", "rmt", "nrmt", "tape", NULL};
 
 /* The list of standard definition files being searched */
 static char *std_databases[] = {
-    "stinit.def",
     "/etc/stinit.def",
     NULL};
 
@@ -234,6 +248,8 @@ find_pars(FILE *dbf, char *company, char *product, char *rev, devdef_tr *defs,
     defs->drive_buffering = (-1);
     defs->timeout = (-1);
     defs->long_timeout = (-1);
+    defs->cleaning = (-1);
+    defs->nowait = (-1);
     for (i=0; i < NBR_MODES; i++) {
 	defs->modedefs[i].defined = FALSE;
 	defs->modedefs[i].blocksize = (-1);
@@ -331,12 +347,14 @@ find_pars(FILE *dbf, char *company, char *product, char *rev, devdef_tr *defs,
 
 	    if ((t = find_string(defstr, "drive-", line, LINEMAX)) != NULL)
 		defs->drive_buffering = strtol(t, NULL, 0);		
-#ifdef MT_ST_SET_TIMEOUT
 	    if ((t = find_string(defstr, "timeout", line, LINEMAX)) != NULL)
 		defs->timeout = strtol(t, NULL, 0);
 	    if ((t = find_string(defstr, "long-time", line, LINEMAX)) != NULL)
 		defs->long_timeout = strtol(t, NULL, 0);
-#endif
+	    if ((t = find_string(defstr, "clean", line, LINEMAX)) != NULL)
+		defs->cleaning = strtol(t, NULL, 0);
+	    if ((t = find_string(defstr, "no-w", line, LINEMAX)) != NULL)
+		defs->nowait = strtol(t, NULL, 0);
 
 	    defs->modedefs[mode].defined = TRUE;
 	    if ((t = find_string(defstr, "block", line, LINEMAX)) != NULL)
@@ -365,10 +383,8 @@ find_pars(FILE *dbf, char *company, char *product, char *rev, devdef_tr *defs,
 		defs->modedefs[mode].can_partitions = strtol(t, NULL, 0);
 	    if ((t = find_string(defstr, "scsi2", line, LINEMAX)) != NULL)
 		defs->modedefs[mode].scsi2logical = strtol(t, NULL, 0);
-#ifdef MT_ST_SYSV
 	    if ((t = find_string(defstr, "sysv", line, LINEMAX)) != NULL)
 		defs->modedefs[mode].sysv = strtol(t, NULL, 0);
-#endif
 	    if ((t = find_string(defstr, "defs-for-w", line, LINEMAX)) != NULL)
 		defs->modedefs[mode].defs_for_writes = strtol(t, NULL, 0);
 
@@ -476,15 +492,15 @@ do_inquiry(char *tname, char *company, char *product, char *rev, int print_non_f
 }
 
 
-
-
 	static int
 tapenum(char *name)
 {
     int dev;
     struct dirent *dent;
     DIR *dirp;
-    char *dn, **dnp, tmpname[PATH_MAX];
+    char tmpname[PATH_MAX];
+    const char *dn;
+    const devdir *dvd;
     struct stat statbuf;
 
     if (strchr(name, '/') != NULL) {  /* Complete name */
@@ -499,14 +515,13 @@ tapenum(char *name)
 	return dev;
     }
     else { /* Search from the device directories */
-	for (dnp=devdirs; *dnp != NULL; dnp++) {
-	    dn = *dnp;
+	for (dvd=devdirs; dvd->dir != NULL; dvd++) {
+	    dn = dvd->dir;
 	    if ((dirp = opendir(dn)) == NULL)
 	      continue;
 
 	    for ( ; (dent = readdir(dirp)) != NULL; )
 		if (!strcmp(dent->d_name, name)) {
-		    dev = minor(statbuf.st_rdev);
 		    strcpy(tmpname, dn);
 		    strcat(tmpname, "/");
 		    strcat(tmpname, dent->d_name);
@@ -529,6 +544,18 @@ tapenum(char *name)
 }
 
 
+	static int
+accept_tape_name(char *name)
+{
+    char **npp;
+
+    for (npp=tape_name_bases; *npp; npp++)
+	if (!strncmp(name, *npp, strlen(*npp)))
+	    return TRUE;
+    return FALSE;
+}
+
+
 	static int
 find_devfiles(int tapeno, char **names)
 {
@@ -536,7 +563,10 @@ find_devfiles(int tapeno, char **names)
     int non_rew[NBR_MODES];
     struct dirent *dent;
     DIR *dirp;
-    char *dn, **dnp, tmpname[PATH_MAX];
+    char tmpname[PATH_MAX], devname[PATH_MAX];
+    const char *dn;
+    const devdir *dvd;
+    devdir tmpdevdirs[2];
     struct stat statbuf;
 
     for (found=0; found < NBR_MODES; found++) {
@@ -544,12 +574,29 @@ find_devfiles(int tapeno, char **names)
 	non_rew[found] = FALSE;
     }
 
-    for (dnp=devdirs, found=0; found < NBR_MODES && *dnp != NULL; dnp++) {
-	dn = *dnp;
+    dvd = devdirs;
+    strcpy(tmpname, DEVFS_PATH);
+    if (!stat(tmpname, &statbuf)) {
+	if (S_ISDIR(statbuf.st_mode)) {  /* Assume devfs, one directory for each tape */
+	    sprintf(devname, DEVFS_TAPEFMT, tapeno);
+	    tmpdevdirs[0].dir = devname;
+	    tmpdevdirs[0].selective_scan = FALSE;
+	    tmpdevdirs[1].dir = NULL;
+	    dvd = &tmpdevdirs[0];
+	}
+    }
+
+    for (found=0; found < NBR_MODES && dvd->dir != NULL; dvd++) {
+	dn = dvd->dir;
 	if ((dirp = opendir(dn)) == NULL)
 	    continue;
 
 	for ( ; (dent = readdir(dirp)) != NULL; ) {
+	    if (!strcmp (dent->d_name, ".") || !strcmp (dent->d_name, ".."))
+		continue;
+	    /* Ignore non-tape devices to avoid loading all the modules */
+	    if (dvd->selective_scan && !accept_tape_name(dent->d_name))
+		continue;
 	    strcpy(tmpname, dn);
 	    strcat(tmpname, "/");
 	    strcat(tmpname, dent->d_name);
@@ -606,18 +653,17 @@ set_defs(devdef_tr *defs, char **fnames)
 		op.mt_op = MTSETDRVBUFFER;
 		op.mt_count = MT_ST_DEF_DRVBUFFER | defs->drive_buffering;
 		if (ioctl(tape, MTIOCTOP, &op) != 0) {
-		    fprintf(stderr, "Can't set drive buffering to %d for mode %d.\n",
-			    defs->modedefs[i].blocksize, i);
+		    fprintf(stderr, "Can't set drive buffering to %d.\n",
+			    defs->drive_buffering);
 		}
 	    }
 
-#ifdef MT_ST_SET_TIMEOUT
 	    if (defs->timeout >= 0) {
 		op.mt_op = MTSETDRVBUFFER;
 		op.mt_count = MT_ST_SET_TIMEOUT | defs->timeout;
 		if (ioctl(tape, MTIOCTOP, &op) != 0) {
-		    fprintf(stderr, "Can't set device timeout %d s for mode %d.\n",
-			    defs->modedefs[i].blocksize, i);
+		    fprintf(stderr, "Can't set device timeout %d s.\n",
+			    defs->timeout);
 		}
 	    }
 
@@ -625,16 +671,26 @@ set_defs(devdef_tr *defs, char **fnames)
 		op.mt_op = MTSETDRVBUFFER;
 		op.mt_count = MT_ST_SET_LONG_TIMEOUT | defs->long_timeout;
 		if (ioctl(tape, MTIOCTOP, &op) != 0) {
-		    fprintf(stderr, "Can't set device long timeout %d s for mode %d.\n",
-			    defs->modedefs[i].blocksize, i);
+		    fprintf(stderr, "Can't set device long timeout %d s.\n",
+			    defs->long_timeout);
+		}
+	    }
+
+	    if (defs->cleaning >= 0) {
+		op.mt_op = MTSETDRVBUFFER;
+		op.mt_count = MT_ST_SET_CLN | defs->cleaning;
+		if (ioctl(tape, MTIOCTOP, &op) != 0) {
+		    fprintf(stderr, "Can't set cleaning request parameter to %x\n",
+			    defs->cleaning);
 		}
 	    }
-#endif
 	}
 
 	op.mt_op = MTSETDRVBUFFER;
 
 	clear_set[0] = clear_set[1] = 0;
+	if (defs->nowait >= 0)
+	    clear_set[defs->nowait != 0] |= MT_ST_NOWAIT;
 	if (defs->modedefs[i].buffer_writes >= 0)
 	    clear_set[defs->modedefs[i].buffer_writes != 0] |= MT_ST_BUFFER_WRITES;
 	if (defs->modedefs[i].async_writes >= 0)
@@ -655,10 +711,8 @@ set_defs(devdef_tr *defs, char **fnames)
 	    clear_set[defs->modedefs[i].can_partitions != 0] |= MT_ST_CAN_PARTITIONS;
 	if (defs->modedefs[i].scsi2logical >= 0)
 	    clear_set[defs->modedefs[i].scsi2logical != 0] |= MT_ST_SCSI2LOGICAL;
-#ifdef MT_ST_SYSV
 	if (defs->modedefs[i].sysv >= 0)
 	    clear_set[defs->modedefs[i].sysv != 0] |= MT_ST_SYSV;
-#endif
 	if (defs->modedefs[i].defs_for_writes >= 0)
 	    clear_set[defs->modedefs[i].defs_for_writes != 0] |= MT_ST_DEF_WRITES;
 
@@ -831,7 +885,8 @@ main(int argc, char **argv)
     else {  /* Initialize all SCSI tapes */
 	for (tapeno=0; tapeno < MAX_TAPES; tapeno++)
 	    if (!define_tape(tapeno, dbf, &defs, FALSE)) {
-		fprintf(stderr, "Initialized %d tape devices.\n", tapeno);
+		fprintf(stderr, "Initialized %d tape device%s.\n",
+			tapeno, (tapeno != 1 ? "s" : ""));
 		return 0;  /* Process tapes until failure */
 	    }
     }
